@@ -1,7 +1,7 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2010 by Dominique Dumont, Krzysztof Tyszecki.
+# This software is Copyright (c) 2012 by Dominique Dumont, Krzysztof Tyszecki.
 #
 # This is free software, licensed under:
 #
@@ -27,8 +27,8 @@
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 
 package Config::Model::DumpAsData;
-BEGIN {
-  $Config::Model::DumpAsData::VERSION = '1.226';
+{
+  $Config::Model::DumpAsData::VERSION = '2.005';
 }
 use Carp;
 use strict;
@@ -43,24 +43,60 @@ Config::Model::DumpAsData - Dump configuration content as a perl data structure
 
 =head1 VERSION
 
-version 1.226
+version 2.005
 
 =head1 SYNOPSIS
 
  use Config::Model ;
+ use Log::Log4perl qw(:easy) ;
+ use Data::Dumper ;
 
- # create your config model
- my $model = Config::Model -> new ;
- $model->create_config_class( ... ) ;
+ Log::Log4perl->easy_init($WARN);
 
- # create instance
- my $inst = $model->instance (root_class_name => 'FooBar', 
-			      instance_name => 'test1');
+ # define configuration tree object
+ my $model = Config::Model->new ;
+ $model ->create_config_class (
+    name => "MyClass",
+    element => [ 
+        [qw/foo bar/] => { 
+            type => 'leaf',
+            value_type => 'string'
+        },
+        baz => { 
+            type => 'hash',
+            index_type => 'string' ,
+            cargo => {
+                type => 'leaf',
+                value_type => 'string',
+            },
+        },
+        
+    ],
+ ) ;
 
- # create root of config
- my $root = $inst -> config_root ;
+ my $inst = $model->instance(root_class_name => 'MyClass' );
 
- my $data =  $root->dump_as_data ;
+ my $root = $inst->config_root ;
+
+ # put some data in config tree the hard way
+ $root->fetch_element('foo')->store('yada') ;
+ $root->fetch_element('bar')->store('bla bla') ;
+ $root->fetch_element('baz')->fetch_with_id('en')->store('hello') ;
+
+ # put more data the easy way
+ my $step = 'baz:fr=bonjour baz:hr="dobar dan"';
+ $root->load( step => $step ) ;
+
+ print Dumper($root->dump_as_data);
+ # $VAR1 = {
+ #         'bar' => 'bla bla',
+ #         'baz' => {
+ #                    'en' => 'hello',
+ #                    'fr' => 'bonjour',
+ #                    'hr' => 'dobar dan'
+ #                  },
+ #         'foo' => 'yada'
+ #       };
 
 =head1 DESCRIPTION
 
@@ -108,7 +144,7 @@ configuration data will be used by the application. (default is yes)
 
 =item skip_auto_write
 
-Skip node that have a C<perl write> capabality in their model. See
+Skip node that have a C<perl write> capability in their model. See
 L<Config::Model::AutoRead>.
 
 =item auto_vivify
@@ -122,7 +158,7 @@ By default, ordered hash (i.e. the order of the keys are important)
 are dumped as Perl list. This is the faster way to dump such hashed
 while keeping the key order. But it's the less readable way. 
 
-When this parmeter is 1 (default), the ordered hash is dumped as a
+When this parameter is 1 (default), the ordered hash is dumped as a
 list:
 
   [ A => 'foo', B => 'bar', C => 'baz' ]
@@ -143,7 +179,7 @@ sub dump_as_data {
 
     my %args = @_;
     my $dump_node = delete $args{node} 
-      || croak "dumpribe: missing 'node' parameter";
+      || croak "dump_as_data: missing 'node' parameter";
     my $full = delete $args{full_dump} ;
     $full = 1 unless defined $full ;
     my $skip_aw = delete $args{skip_auto_write} || '' ;
@@ -175,8 +211,8 @@ sub dump_as_data {
 	    my $v ;
 	    $scanner->scan_hash(\$v,$node,$element_name,$_);
 	    # create the key even if $v is undef 
-	    $h{$_} = $v ;
-	    ( $_ , $v ) ;
+	    $h{$_} = $v if defined $v;
+	    defined $v ? ( $_ , $v ) : ();
 	} @keys ;
 
 	my $ordered_hash = $node->fetch_element($element_name)->ordered ;
@@ -265,6 +301,117 @@ sub dump_as_data {
     return $result ;
 }
 
+=head1 Methods
+
+=head2 dump_annotations_as_pod(...)
+
+Return a string formatted in pod (See L<perlpod>) with the annotations.
+
+Parameters are:
+
+=over
+
+=item node
+
+Reference to a L<Config::Model::Node> object. Mandatory
+
+=item experience
+
+master, advanced or beginner
+
+=item check_list
+
+Yes, no or skip
+
+=back
+
+=cut 
+
+sub dump_annotations_as_pod {
+    my $self = shift ;
+
+    my %args = @_;
+    my $dump_node = delete $args{node} 
+      || croak "dump_annotations_as_pod: missing 'node' parameter";
+   
+    my $annotation_to_pod = sub {
+        my $obj = shift ;
+        my $path = shift || $obj->location;
+	my $a = $obj->annotation ;
+	if ($a) {
+	    chomp $a ;
+	    return "=item $path\n\n$a\n\n" ;
+        }
+        else {
+            return '';
+        }
+    };
+    
+    my $std_cb = sub {
+        my ( $scanner, $data_r, $obj, $element, $index, $value_obj ) = @_;
+        $$data_r .= $annotation_to_pod->($value_obj) ; 
+    };
+
+    my $hash_element_cb = sub {
+        my ($scanner, $data_ref,$node,$element_name,@keys) = @_ ;
+        my $h = $node->fetch_element($element_name) ;
+        my $h_path = $h->location . ':';
+	foreach (@keys) { 
+	    $$data_ref .= $annotation_to_pod->(
+                $h->fetch_with_id($_),
+                $h_path.$_
+            );
+	    $scanner->scan_hash($data_ref,$node,$element_name,$_) ;
+	}
+    } ;
+    
+    my $node_content_cb = sub {
+	my ($scanner, $data_ref,$node,@element) = @_ ;
+	my $node_path = $node->location ;
+	$node_path .= ' ' if $node_path ;
+	foreach (@element) { 
+	    $$data_ref .= $annotation_to_pod->(
+                $node->fetch_element(name => $_, check => 'no'),
+                $node_path.$_
+            );
+	    $scanner->scan_element($data_ref, $node,$_) ;
+	}
+    };
+
+    my @scan_args = (
+		     experience            => delete $args{experience} || 'master',
+		     check                 => delete $args{check} || 'yes' ,
+		     fallback              => 'all',
+		     leaf_cb               => $std_cb ,
+		     node_content_cb       => $node_content_cb,
+		     hash_element_cb       => $hash_element_cb ,
+		     list_element_cb       => $hash_element_cb ,
+		    );
+
+    my @left = keys %args;
+    croak "dump_annotations_as_pod: unknown parameter:@left" if @left;
+
+    # perform the scan
+    my $view_scanner = Config::Model::ObjTreeScanner->new(@scan_args);
+
+    my $obj_type = $dump_node->get_type ;
+    my $result = '' ;
+
+    my $a = $dump_node->annotation ;
+    my $l = $dump_node->location ;
+    $result .= "=item $l\n\n$a\n\n" if $a ;
+
+    if ($obj_type =~ /node/) {
+	$view_scanner->scan_node(\$result ,$dump_node);
+    }
+    else {
+	croak "dump_annotations_as_pod: unexpected type: $obj_type";
+    }
+
+    return '' unless $result ;
+    return "=head1 Annotations\n\n=over\n\n".$result."=back\n\n" ;
+}
+
 1;
 
 =head1 AUTHOR
@@ -273,4 +420,4 @@ Dominique Dumont, (ddumont at cpan dot org)
 
 =head1 SEE ALSO
 
-L<Config::Model>,L<Config::Model::Node>,L<Config::Model::ObjTreeScanner>
\ No newline at end of file
+L<Config::Model>,L<Config::Model::Node>,L<Config::Model::ObjTreeScanner>
