@@ -1,35 +1,22 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2011 by Dominique Dumont, Krzysztof Tyszecki.
+# This software is Copyright (c) 2012 by Dominique Dumont, Krzysztof Tyszecki.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
-#    Copyright (c) 2005-2011 Dominique Dumont.
-#
-#    This file is part of Config-Model.
-#
-#    Config-Model is free software; you can redistribute it and/or
-#    modify it under the terms of the GNU Lesser Public License as
-#    published by the Free Software Foundation; either version 2.1 of
-#    the License, or (at your option) any later version.
-#
-#    Config-Model is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#    Lesser Public License for more details.
-#
-#    You should have received a copy of the GNU Lesser Public License
-#    along with Config-Model; if not, write to the Free Software
-#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-
 package Config::Model::Instance;
 {
-  $Config::Model::Instance::VERSION = '1.260';
+  $Config::Model::Instance::VERSION = '2.005';
 }
-use Scalar::Util qw(weaken) ;
+#use Scalar::Util qw(weaken) ;
+
+use Any::Moose ;
+use namespace::autoclean;
+use Any::Moose '::Util::TypeConstraints';
+
 use File::Path;
 use Log::Log4perl qw(get_logger :levels);
 
@@ -39,24 +26,375 @@ use Config::Model::Node ;
 use Config::Model::Loader;
 use Config::Model::SearchElement;
 use Config::Model::Iterator;
+use Config::Model::ObjTreeScanner;
 
-use strict ;
-use Carp;
 use warnings FATAL => qw(all);
 use warnings::register ;
 
 
-use Carp qw/croak confess cluck/;
+use Carp qw/carp croak confess cluck/;
 
 my $logger = get_logger("Instance") ;
 
+has [qw/root_class_name/] => (is => 'ro', isa => 'Str', required => 1) ;
+
+has config_model => (
+    is => 'ro', 
+    isa => 'Config::Model', 
+    weak_ref => 1 ,
+    required => 1
+) ;
+
+has check => ( 
+    is => 'ro', 
+    isa => 'Str', 
+    default => 'yes',
+    reader => 'read_check' ,
+) ;
+
+# a unique (instance wise) placeholder for various tree objects
+# to store information
+has _safe => (
+    is => 'rw',
+    isa => 'HashRef',
+    traits => ['Hash'] ,
+    default => sub { {} } ,
+    handles => {
+        data => 'accessor' ,
+    },
+) ;
+
+# preset mode:  to load values found by HW scan or other automatic scheme
+# layered mode: to load values found in included files (e.g. a la multistrap)
+has [qw/preset layered/] => (
+    is => 'ro',
+    isa => 'Bool' ,
+    default => 0,
+);
+
+has needs_save => (
+    is => 'rw',
+    isa => 'Bool' ,
+    default => 0,
+);
+
+# initial_load mode: when data is loaded the first time
+has initial_load => ( 
+    is => 'ro',
+    isa => 'Bool' ,
+    default => 1,
+) ;
+
+# This array holds a set of sub ref that will be invoked when
+# the users requires to write all configuration tree in their
+# backend storage.
+has _write_back => (
+    is => 'ro',
+    isa => 'ArrayRef',
+    traits => ['Array'],
+    handles => {
+        register_write_back => 'push' ,
+        count_write_back => 'count' , # mostly for tests
+    },
+    default => sub { [] } ,
+);
+
+# used for auto_read auto_write feature
+has [qw/name root_dir config_file backend/] => (
+    is => 'ro',
+    isa => 'Maybe[Str]' ,
+);
+has skip_read => (is => 'ro', isa => 'Bool', default => 0) ;
+
+sub BUILD {
+    my $self = shift ;
+
+    #carp "instance new: force_load is deprecated" if defined $args{force_load} ;
+    #$read_check = 'no' if delete $args{force_load} ;
+
+    # cleanup paths
+    map { $self->{$_} .= '/' if defined $self->{$_} and $self->{$_} !~ m!/$!}
+      qw/root_dir/;
+}
+
+has tree => (
+    is => 'ro',
+    isa => 'Config::Model::Node',
+    builder => 'reset_config' ,
+    reader => 'config_root' ,
+);
+
+sub reset_config {
+    my $self =shift;
+
+    return Config::Model::Node->new(
+        config_class_name => $self->{root_class_name},
+        instance          => $self,
+        container         => $self,
+        skip_read         => $self->{skip_read},
+        config_file       => $self->{config_file} ,
+    );
+}
+
+sub preset_start {
+    my $self = shift ;
+    $logger->info("Starting preset mode");
+    carp "Cannot start preset mode during layered mode" 
+        if $self->{layered} ;
+    $self->{preset} = 1;
+}
+
+
+sub preset_stop {
+    my $self = shift ;
+    $logger->info("Stopping preset mode");
+    $self->{preset} = 0;
+}
+
+sub preset_clear {
+    my $self = shift ;
+
+    my $leaf_cb = sub {
+        my ($scanner, $data_ref,$node,$element_name,$index, $leaf_object) = @_ ;
+        $leaf_object->clear_preset ;
+    } ;
+    
+    $self->_stuff_clear($leaf_cb) ;
+}
+
+
+sub layered_start {
+    my $self = shift ;
+    $logger->info("Starting layered mode");
+    carp "Cannot start layered mode during preset mode" 
+        if $self->{preset} ;
+    $self->{layered} = 1;
+}
+
+
+sub layered_stop {
+    my $self = shift ;
+    $logger->info("Stopping layered mode");
+    $self->{layered} = 0;
+}
+
+
+sub layered_clear {
+    my $self = shift ;
+    
+    my $leaf_cb = sub {
+        my ($scanner, $data_ref,$node,$element_name,$index, $leaf_object) = @_ ;
+        $$data_ref ||= $leaf_object->clear_layered ;
+    };
+
+    $self->_stuff_clear($leaf_cb);
+}
+
+
+sub get_data_mode {
+    my $self = shift;
+    return
+        $self->{layered} ? 'layered'
+      : $self->{preset}  ? 'preset'
+      :                    'normal';
+}
+
+
+sub _stuff_clear {
+    my ($self,$leaf_cb) = @_ ;
+    
+    # this sub may remove hash keys that were entered by user if the 
+    # corresponding hash value has no data. 
+    # it also clear auto_created ids if there's no data in there
+    my $h_cb = sub {
+        my  ($scanner, $data_ref,$node,$element_name,@keys) = @_ ;
+        my $obj = $node->fetch_element($element_name) ;
+        
+        foreach my $k (@keys) {
+            my $has_data = 0;
+            $scanner->scan_hash(\$has_data,$node,$element_name,$k);
+            $obj->remove($k) unless $has_data ;
+            $$data_ref ||= $has_data ;
+        }
+    };
+    
+    my $wiper = Config::Model::ObjTreeScanner->new (
+        fallback => 'all',
+        auto_vivify => 0,
+        check => 'skip' ,
+        leaf_cb => $leaf_cb ,
+        hash_element_cb => $h_cb,
+        list_element_cb => $h_cb,
+    );
+
+    $wiper->scan_node(undef,$self->config_root) ;
+
+}
+
+sub initial_load_start {
+    my $self = shift ;
+    $logger->info("Start initial_load mode");
+    $self->{initial_load} = 1;
+}
+
+sub initial_load_stop {
+    my $self = shift ;
+    $logger->info("Stopping initial_load mode");
+    $self->{initial_load} = 0;
+}
+
+
+sub load {
+    my $self = shift ;
+    my $loader = Config::Model::Loader->new ;
+    my %args = @_ eq 1 ? (step => $_[0]) : @_ ;
+    $loader->load(node => $self->{tree}, %args) ;
+}
+
+
+sub search_element {
+    my $self = shift ;
+    $self->{tree}->search_element(@_) ;
+}
+
+
+sub wizard_helper {
+    carp __PACKAGE__,"::wizard_helper helped is deprecated. Call iterator instead" ;
+    goto &iterator ;
+}
+
+
+sub iterator {
+    my $self = shift ;
+    my @args = @_ ;
+
+    my $tree_root = $self->config_root ;
+
+    return Config::Model::Iterator->new ( root => $tree_root, @args) ;
+}
+
+
+sub read_directory {
+    carp "read_directory is deprecated";
+    return shift -> {root_dir} ;
+}
+
+sub read_root_dir {
+    return shift -> {root_dir} ;
+}
+
+sub write_directory {
+    my $self = shift ;
+    carp "write_directory is deprecated";
+    return $self -> {root_dir} ;
+}
+
+sub write_root_dir {
+    my $self = shift ;
+    return $self -> {root_dir} ;
+}
+
+
+# FIXME: record changes to implement undo/redo ?
+sub notify_change {
+    my $self = shift ;
+    $self->{needs_save} = 1;
+}
+
+
+sub write_back {
+    my $self = shift ;
+    my %args = scalar @_ > 1  ? @_ 
+             : scalar @_ == 1 ? (config_dir => $_[0]) 
+	     :                  () ; 
+
+    my $force_backend = delete $args{backend} || $self->{backend} ;
+    my $force_write = delete $args{force} || 0;
+ 
+    foreach (keys %args) {
+        if (/^(root|config_dir)$/) {
+            $args{$_} ||= '' ;
+            $args{$_} .= '/' if $args{$_} and $args{$_} !~ m(/$) ;
+        }
+        elsif (not /^config_file$/) {
+            croak "write_back: wrong parameters $_" ;
+        }
+     }
+
+    croak "write_back: no subs registered in instance $self->{name}. cannot save data\n" 
+      unless @{$self->{_write_back}} ;
+
+    foreach my $path (@{$self->{_write_back}}) {
+	$logger->info("write_back called on node $path");
+        my $node = $self->config_root->grab(step => $path, type => 'node');
+        $node->write_back(
+            %args, 
+            config_file => $self->{config_file} ,
+            backend => $force_backend,
+            force => $force_write,
+        );
+    }
+    $self->needs_save(0) ;
+}
+
+
+sub apply_fixes {
+    my $self = shift ;
+
+    # define leaf call back
+    my $fix_leaf = sub { 
+      my ($scanner, $data_ref, $node,$element_name,$index, $leaf_object) = @_ ;
+      $leaf_object->apply_fixes ;
+    } ;
+
+    my $fix_hash = sub {
+        my ( $scanner, $data_r, $node, $element, @keys ) = @_;
+
+        return unless @keys;
+
+        # leaves must be fixed before the hash, hence the 
+        # calls to scan_hash before apply_fixes
+        map {$scanner->scan_hash($data_r,$node,$element,$_)} @keys ;
+
+        $node->fetch_element($element)->apply_fixes ;
+    } ;
+    
+    my $fix_list = sub {
+        my ( $scanner, $data_r, $node, $element, @keys ) = @_;
+
+        return unless @keys;
+
+        map {$scanner->scan_list($data_r,$node,$element,$_)} @keys ;
+        $node->fetch_element($element)->apply_fixes ;
+    } ;
+    
+   my $scan = Config::Model::ObjTreeScanner-> new ( 
+        hash_element_cb => $fix_hash ,
+        list_element_cb => $fix_list ,
+        leaf_cb => $fix_leaf ,
+        check => 'no',
+    ) ;
+
+    $scan->scan_node(undef, $self->config_root) ;
+}
+
+__PACKAGE__->meta->make_immutable;
+
+1;
+
+
+
+__END__
+
+=pod
+
 =head1 NAME
 
 Config::Model::Instance - Instance of configuration tree
 
 =head1 VERSION
 
-version 1.260
+version 2.005
 
 =head1 SYNOPSIS
 
@@ -137,229 +475,93 @@ If you need to load configuration data that are not correct, you can
 use C<< force_load => 1 >>. Then, wrong data will be discarded (equivalent to 
 C<check => 'no'> ).
 
-=cut
-
-sub new {
-    my $proto = shift ;
-    my $class = ref($proto) || $proto ;
-    my %args = @_ ;
-
-    my $root_class_name = delete $args{root_class_name} || 
-      confess __PACKAGE__," error: missing root_class_name parameter" ;
-
-    my $config_model = delete $args{config_model} || 
-      confess __PACKAGE__," error: missing config_model parameter" ;
-
-    confess __PACKAGE__," error: config_model is not a Config::Model object"
-      unless $config_model->isa('Config::Model') ; 
-
-    my $read_check = delete $args{check} || 'yes' ;
-    carp "instance new: force_load is deprecated" if defined $args{force_load} ;
-    $read_check = 'no' if delete $args{force_load} ;
-
-    my $self 
-      = {
-	 # stack used to store whether read and/or write check must 
-	 # be done in tree objects (Value, Id ...)
-	 check_stack => [ { fetch => 1,
-			    store => 1,
-			    type  => 1 } ],
-
-	 # a unique (instance wise) placeholder for various tree objects
-	 # to store information
-	 safe => {
-		 } ,
-
-        read_check => $read_check ,
-
-	 # preset mode to load values found by HW scan or other
-	 # automatic scheme
-	 preset => 0,
-
-	 config_model => $config_model ,
-	 root_class_name => $root_class_name ,
-
-	 # This array holds a set of sub ref that will be invoked when
-	 # the users requires to write all configuration tree in their
-	 # backend storage.
-	 write_back => [] ,
-
-	 # used for auto_read auto_write feature
-	 name            =>  delete $args{name} ,
-	 root_dir        =>  delete $args{root_dir},
-
-	 backend         =>  delete $args{backend} || '',
-	 skip_read       =>  delete $args{skip_read} || 0,
-	};
-
-    my @left = keys %args ;
-    croak "Instance->new: unexpected parameter: @left" if @left ;
-
-    # cleanup paths
-    map { $self->{$_} .= '/' if defined $self->{$_} and $self->{$_} !~ m!/$!}
-      qw/root_dir/;
-
-    weaken($self->{config_model}) ;
-
-    bless $self, $class;
-
-    $self->reset_config() ;
-
-    return $self ;
-}
-
-
 =head1 METHODS
 
 =head2 name()
 
 Returns the instance name.
 
-=cut
-
-sub name {
-    return shift->{name} ;
-}
-
 =head2 config_root()
 
 Returns the root object of the configuration tree.
 
-=cut
-
-sub config_root {
-    return shift->{tree} ;
-}
-
 =head2 read_check()
 
 Returns how to check read files.
 
-=cut
-
-sub read_check {
-    return shift->{read_check} ;
-}
-
 =head2 reset_config
 
 Destroy current configuration tree (with data) and returns a new tree with
 data (and annotations) loaded from disk.
 
-=cut
-
-sub reset_config {
-    my ($self, %args ) = @_ ;
-
-    $self->{tree} = Config::Model::Node
-      -> new ( config_class_name => $self->{root_class_name},
-	       instance => $self,
-	       config_model => $self->{config_model},
-	       skip_read  => $self->{skip_read},
-	     );
-
-    # $self->{annotation_saver} = Config::Model::Annotation
-    #   -> new (
-    # 	      config_class_name => $self->{root_class_name},
-    # 	      instance => $self ,
-    # 	      root_dir => $self->{root_dir} ,
-    # 	     ) ;
-    # $self->{annotation_saver}->load ;
-
-    return $self->{tree} ;
-}
-
-
 =head2 config_model()
 
 Returns the model (L<Config::Model> object) of the configuration tree.
 
-=cut
-
-sub config_model {
-    return shift->{config_model} ;
-}
-
 =head2 annotation_saver()
 
 Returns the object loading and saving annotations. See
 L<Config::Model::Annotation> for details.
 
-=cut
-
-# sub annotation_saver {
-#     my $self = shift ;
-#     return $self->{annotation_saver} ;
-#}
-
 =head2 preset_start ()
 
 All values stored in preset mode are shown to the user as default
 values. This feature is useful to enter configuration data entered by
 an automatic process (like hardware scan)
 
-=cut
-
-sub preset_start {
-    my $self = shift ;
-    $logger->info("Starting preset mode");
-    $self->{preset} = 1;
-}
-
 =head2 preset_stop ()
 
 Stop preset mode
 
-=cut
-
-sub preset_stop {
-    my $self = shift ;
-    $logger->info("Stopping preset mode");
-    $self->{preset} = 0;
-}
-
 =head2 preset ()
 
 Get preset mode
 
-=cut
+=head2 preset_clear()
 
-sub preset {
-    my $self = shift ;
-    return $self->{preset} ;
-}
+Clear all preset values stored.
+
+=head2 layered_start ()
+
+All values stored in layered mode are shown to the user as default
+values. This feature is useful to enter configuration data entered by
+an automatic process (like hardware scan)
+
+=head2 layered_stop ()
+
+Stop layered mode
+
+=head2 layered ()
+
+Get layered mode
+
+=head2 layered_clear()
+
+Clear all layered values stored.
+
+=head2 get_data_mode 
+
+Returns 'normal' or 'preset' or 'layered'. Does not take into account initial_load.
+
+=head2 initial_load_stop ()
+
+Stop initial_load mode. Instance is built with initial_load as 1. Read backend
+will clear this value once the first read is done.
+
+=head2 initial_load ()
+
+Get initial_load mode
 
 =head2 data( kind, [data] )
 
 The data method provide a way to store some arbitrary data in the
 instance object.
 
-=cut
-
-sub data {
-    my $self = shift;
-    my $kind = shift || croak "undefined data kind";
-    my $store = shift ;
-
-    $self->{safe}{$kind} = $store if defined $store;
-    return $self->{safe}{$kind} ;
-}
-
-
 =head2 load( "..." )
 
 Load configuration tree with configuration data. See
 L<Config::Model::Loader> for more details
 
-=cut
-
-sub load {
-    my $self = shift ;
-    my $loader = Config::Model::Loader->new ;
-    my %args = @_ eq 1 ? (step => $_[0]) : @_ ;
-    $loader->load(node => $self->{tree}, %args) ;
-}
-
 =head2 searcher ( )
 
 Returns an object dedicated to search an element in the configuration
@@ -368,24 +570,10 @@ model (respecting privilege level).
 This method returns a L<Config::Model::Searcher> object. See
 L<Config::Model::Searcher> for details on how to handle a search.
 
-=cut
-
-sub search_element {
-    my $self = shift ;
-    $self->{tree}->search_element(@_) ;
-}
-
 =head2 wizard_helper ( ... )
 
 Deprecated. Call L</iterator> instead.
 
-=cut
-
-sub wizard_helper {
-    carp __PACKAGE__,"::wizard_helper helped is deprecated. Call iterator instead" ;
-    goto &iterator ;
-}
-
 =head2 iterator 
 
 This method returns a L<Config::Model::Iterator> object. See
@@ -394,17 +582,6 @@ L<Config::Model::Iterator> for details.
 Arguments are explained in  L<Config::Model::Iterator>
 L<constructor arguments|Config::Model::Iterator/"Creating an iterator">.
 
-=cut
-
-sub iterator {
-    my $self = shift ;
-    my @args = @_ ;
-
-    my $tree_root = $self->config_root ;
-
-    return Config::Model::Iterator->new ( root => $tree_root, @args) ;
-}
-
 =head2 
 
 =head1 Auto read and write feature
@@ -420,58 +597,23 @@ L<Config::Model::AutoRead> for details.
 
 Returns root directory where configuration data is read from.
 
-=cut
-
-sub read_directory {
-    carp "read_directory is deprecated";
-    return shift -> {root_dir} ;
-}
-
-sub read_root_dir {
-    return shift -> {root_dir} ;
-}
-
 =head2 backend()
 
 Get the preferred backend method for this instance (as passed to the
 constructor).
 
-=cut
-
-sub backend {
-    return shift -> {backend} ;
-}
-
 =head2 write_root_dir()
 
 Returns root directory where configuration data is written to.
 
-=cut
-
-sub write_directory {
-    my $self = shift ;
-    carp "write_directory is deprecated";
-    return $self -> {root_dir} ;
-}
-
-sub write_root_dir {
-    my $self = shift ;
-    return $self -> {root_dir} ;
-}
-
 =head2 register_write_back ( node_location )
 
 Register a node path that will be called back with
 C<write_back> method.
 
-=cut
+=head2 notify_change
 
-sub register_write_back {
-    my ($self,$node_path) = @_ ;
-    $logger->debug("register_write_back: instance '$self->{name}' registers node '$node_path'") ;
-
-    push @{$self->{write_back}}, $node_path ;
-}
+Notify that some data has changed in the tree. 
 
 =head2 write_back ( ... )
 
@@ -491,86 +633,11 @@ C<< backend => 'all' >>.
 
 C<write_back> will croak if no write call-back are known.
 
-=cut
-
-sub write_back {
-    my $self = shift ;
-    my %args = scalar @_ > 1  ? @_ 
-             : scalar @_ == 1 ? (config_dir => $_[0]) 
-	     :                  () ; 
-
-    my $force_backend = delete $args{backend} || $self->{backend} ;
-
-    map {croak "write_back: wrong parameters $_" 
-	     unless /^(root|config_dir)$/ ;
-	 $args{$_} ||= '' ;
-	 $args{$_} .= '/' if $args{$_} and $args{$_} !~ m(/$) ;
-     }
-      keys %args;
-
-    croak "write_back: no subs registered in instance $self->{name}. cannot save data\n" 
-      unless @{$self->{write_back}} ;
-
-    foreach my $path (@{$self->{write_back}}) {
-	$logger->info("write_back called on node $path");
-        my $node = $self->config_root->grab(step => $path, type => 'node');
-        $node->write_back(%args, backend => $force_backend);
-    }
-}
-
 =head2 apply_fixes
 
 Scan the tree and apply fixes that are attached to warning specifications. 
 See C<warn_if_match> or C<warn_unless_match> in L<Config::Model::Value/>.
 
-=cut
-
-sub apply_fixes {
-    my $self = shift ;
-
-    # define leaf call back
-    my $fix_leaf = sub { 
-      my ($scanner, $data_ref, $node,$element_name,$index, $leaf_object) = @_ ;
-      $leaf_object->apply_fixes ;
-    } ;
-
-    my $fix_hash = sub {
-        my ( $scanner, $data_r, $node, $element, @keys ) = @_;
-
-        return unless @keys;
-
-        # leaves must be fixed before the hash, hence the 
-        # calls to scan_hash before apply_fixes
-        map {$scanner->scan_hash($data_r,$node,$element,$_)} @keys ;
-
-        $node->fetch_element($element)->apply_fixes ;
-    } ;
-    
-    my $fix_list = sub {
-        my ( $scanner, $data_r, $node, $element, @keys ) = @_;
-
-        return unless @keys;
-
-        map {$scanner->scan_list($data_r,$node,$element,$_)} @keys ;
-        $node->fetch_element($element)->apply_fixes ;
-    } ;
-    
-   my $scan = Config::Model::ObjTreeScanner-> new ( 
-        hash_element_cb => $fix_hash ,
-        list_element_cb => $fix_list ,
-        leaf_cb => $fix_leaf ,
-        check => 'no',
-    ) ;
-
-    $scan->scan_node(undef, $self->config_root) ;
-}
-
-sub push_no_value_check { carp "push_no_value_check is deprecated";}
-sub pop_no_value_check  { carp "pop_no_value_check is deprecated";}
-sub get_value_check { carp "get_value_check is deprecated";}
-
-1;
-
 =head1 AUTHOR
 
 Dominique Dumont, (ddumont at cpan dot org)
