@@ -1,7 +1,7 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2011 by Dominique Dumont, Krzysztof Tyszecki.
+# This software is Copyright (c) 2012 by Dominique Dumont, Krzysztof Tyszecki.
 #
 # This is free software, licensed under:
 #
@@ -30,7 +30,6 @@
       {
         'value_type' => 'uniline',
         'summary' => 'package maintainer\'s name and email address',
-        'match' => '[\\w\\s\\-]+<[\\w\\.\\-\\@]+>',
         'mandatory' => '1',
         'type' => 'leaf',
         'description' => 'The package maintainer\'s name and email address. The name must come first, then the email address inside angle brackets <> (in RFC822 format).
@@ -42,7 +41,6 @@ If the maintainer\'s name contains a full stop then the whole field will not wor
         'cargo' => {
           'replace_follow' => '!Debian::Dpkg meta email-updates',
           'value_type' => 'uniline',
-          'match' => '[\\w\\s\\-]+<[\\w\\.\\-\\@]+>',
           'type' => 'leaf'
         },
         'type' => 'list'
@@ -83,6 +81,7 @@ The archive area and section for each package should be specified in the package
           'class' => 'Config::Model::Debian::Dependency',
           'type' => 'leaf'
         },
+        'duplicates' => 'warn',
         'type' => 'list'
       },
       'Build-Depends-Indep',
@@ -92,6 +91,7 @@ The archive area and section for each package should be specified in the package
           'class' => 'Config::Model::Debian::Dependency',
           'type' => 'leaf'
         },
+        'duplicates' => 'warn',
         'type' => 'list'
       },
       'Build-Conflicts',
@@ -108,8 +108,8 @@ The archive area and section for each package should be specified in the package
         'summary' => 'Debian policy version number this package complies to',
         'warn_unless_match' => {
           '3\\.9\\.2' => {
-            'msg' => 'Current standard version is 3.9.2',
-            'fix' => '$_ = \'3.9.2\';'
+            'msg' => 'Current standards version is 3.9.2',
+            'fix' => '$_ = undef; # restore default value'
           }
         },
         'match' => '\\d+\\.\\d+\\.\\d+(\\.\\d+)?',
@@ -119,9 +119,18 @@ The archive area and section for each package should be specified in the package
       },
       'Vcs-Browser',
       {
+        'compute' => {
+          'use_eval' => '1',
+          'formula' => '$maintainer =~ /pkg-perl/ ? "http://anonscm.debian.org/gitweb/?p=pkg-perl/packages/$pkgname.git" : undef ;',
+          'variables' => {
+            'maintainer' => '- Maintainer',
+            'pkgname' => '- Source'
+          },
+          'allow_override' => '1'
+        },
         'value_type' => 'uniline',
         'summary' => 'web-browsable URL of the VCS repository',
-        'match' => '^http://',
+        'match' => '^https?://',
         'type' => 'leaf',
         'description' => 'Value of this field should be a http:// URL pointing to a web-browsable copy of the Version Control System repository used to maintain the given package, if available.
 
@@ -165,6 +174,15 @@ The information is meant to be useful for a user knowledgeable in the given Vers
       },
       'Vcs-Git',
       {
+        'compute' => {
+          'use_eval' => '1',
+          'formula' => '$maintainer =~ /pkg-perl/ ? "git://git.debian.org/git/pkg-perl/packages/$pkgname.git" : \'\' ;',
+          'variables' => {
+            'maintainer' => '- Maintainer',
+            'pkgname' => '- Source'
+          },
+          'allow_override' => '1'
+        },
         'value_type' => 'uniline',
         'summary' => 'URL of the VCS repository',
         'type' => 'leaf',
